//---------------------------------------------------------------------------

#ifndef OutputShortH
#define OutputShortH

#include "BehaviorBase.h"
#include "Constants.h"

class clSimManager;
class xercesc::DOMDocument;

/**
* Replicates the old-school tab-delimited text output file.  It's a shortcut
* if a detailed output file isn't what you need.  It supports subplots - as
* many as you want.
*
* This will save absolute and relative basal area and density for snags,
* adults, and saplings, and absolute density for seedlings.
*
* The namestring of this behavior, and the name which it is called in the
* behavior list of the parameter file, is "short output".
*
* Copyright 2004 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
* <br>June 24, 2004 - Added snag support (LEM)
* <br>October 25, 2005 - Changed the way we search for trees (LEM)
* <br>January 2, 2007 - Support for user-defined subplot resolution and made
* version 1.1 (LEM) 
*/

class clShortOutput : public clBehaviorBase {

  public:

  /**
  * Constructor.
  *
  * @param p_oSimManager Sim Manager object.
  */
  clShortOutput(clSimManager *p_oSimManager);

  /**
  * Destructor.
  */
  ~clShortOutput();

  /**
  * Collects data and writes the output file.
  */
  void Action();

  protected:

  /**
  * Reads the desired options out of the parameter file.
  *
  * @param p_oDoc DOM tree of the parsed input file.
  */
  void GetData(xercesc::DOMDocument *p_oDoc);

  char m_cFileName[MAX_FILENAME_SIZE]; /**<Output file name.  If it already
  exists, new data is appended at the end*/

  /**
  * Data structure for defining what tree output data to save
  */
  struct stcTreeOutputInfo {
    int iType;     /**<Type number that we're saving for.  From clTree*/
    bool bSaveRBA; /**<Whether to save plot-level relative basal area.  Ignored
                    for seeds and seedlings.*/
    bool bSaveABA; /**<Whether to save plot-level absolute basal area.  Ignored
                   for seeds and seedlings.*/
    bool bSaveRDN; /**<Whether to save plot-level relative density.*/
    bool bSaveADN; /**<Whether to save plot-level absolute density.*/
  } *mp_treeSettings;  /**<An array of stcTreeOutputInfo's, one for each tree
                        type that we're saving data for.*/
  short int m_iNumTypesToSave; /**<Number of tree types that we're saving
  data for.*/

  /**
  * Structure for holding coordinates
  */
  struct stcCoords {int iX; /**<Grid cell X number*/
                    int iY; /**<Grid cell Y number*/
                    };

  /**
  * Data structure for saving subplot data.  They get the same data saved as
  * the plot as a whole.
  */
  struct stcSubplotInfo {
    char cSubplotName[MAX_SUBPLOT_NAME_SIZE]; /**<Name of the subplot so it
    can be identified in output files*/
    stcCoords *p_cellList; /**<Array of grid cell coords*/
    float fArea; /**<Subplot's area, in ha*/
    short int iNumCells; /**<Number of grid cells in this subplot*/
  } *mp_subplots; /**<An array of stcSubplotInfos, one for each subplot to save*/
  short int m_iNumSubplotsToSave; /**<Number of subplots we're saving data for*/
  
  /**Number of plot cells in the X direction - for subplots*/
  short int m_iNumXCells;
  
  /**Number of plot cells in the Y direction - for subplots*/
  short int m_iNumYCells;
  
  /**Length of subplot cells in X direction - defaults to match tree 
   * population*/
  float m_fXCellLength;
  
  /**Length of subplot cells in Y direction - defaults to match tree 
   * population*/
  float m_fYCellLength;

  /**
  * Writes output file header.
  */
  void WriteFileHeader();

  /**
  * Extracts the output data relating to tree saving.
  *
  * @param p_oDoc DOM tree of parsed input file.
  */
  void ExtractTreeInfo(xercesc::DOMDocument *p_oDoc);

  /**
  * Extracts the subplot data.
  *
  * @param p_oDoc DOM tree of parsed input file.
  */
  void ExtractSubplotInfo(xercesc::DOMDocument *p_oDoc);

  /**
  * Tests to see if a given node exists and has an attribute called "save"
  * equal to true.
  *
  * @param p_oParentElement Pointer to the parent element of the node in
  * question.
  * @param cNodeName Tag name of the node.
  * @return True if the save attribute is true; if the node does not exist or
  * is equal to false, false is returned.
  */
  bool TestForSave(DOMElement *p_oParentElement, char *cNodeName);
};
//---------------------------------------------------------------------------
#endif
